/*
    FaxClient V1.2
    --------------

   (c) 1998 Matthias Greim (angus@uni-paderborn.de)

    this is Freeware, just drop me a mail if you like it
    FaxClient was written and tested with JDK1.1.6 + Swing1.0.2
    under Linux and Windows and with JDK1.2b3 under Windows.

    Commandline: Filename of Phonebook, if not given,
                 phonebook.dat ist used in current directory.
    Attention:   There are no checks against wrong faxnumbers
                 (e.g. things like "54df32-df") and wrong
		 times. Faxnumberfield is directly passed to
                 fprintfax, so multible receivers can be
                 entered by separating the numbers with a
                 comma or space. If no time is given and enable
		 is checked, the default delay time is used.
                 Note that after the start the program stays
                 invisible until fprintfax connects to your
                 machine on port 5555.
*/

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import com.sun.java.swing.*;

class PhoneEntry {
  public String fax;
  public String name;

  public String toString() {
    return name;
  }

}

public class FaxClient implements Runnable {
  private JComboBox receiverCB    = new JComboBox();
  private JTextField faxTF        = new JTextField();
  private JTextField loginTF      = new JTextField();
  private JTextField nameTF       = new JTextField();
  private JCheckBox delayEnableCB = new JCheckBox("enable");
  private JTextField delayTimeTF  = new JTextField(5);
  private JButton Add;
  private JButton Remove;
  private JButton Send;
  private JButton Quit;
  
  static JFrame mainFrame;
  static Thread listenThread;
  static Socket socket;
  static String fileName;

  static final int DEFAULT_PORT = 5555;

  /* Initialize the whole application */
  private void init() {
    /* Create Main Window for FaxClient */
    mainFrame = new JFrame("FaxClient");
    mainFrame.setResizable(false);

    /* Create all Labels */
    JLabel l1 = new JLabel("Receiver:");
    JLabel l2 = new JLabel("Fax Number:");
    JLabel l3 = new JLabel("User Login:");
    JLabel l4 = new JLabel("User Name:");
    JLabel l5 = new JLabel("Time:");

    /* Create the four buttons */
    ImageIcon sendImage = new ImageIcon("send.gif");
    Send = new JButton("Send", sendImage);
    Send.setHorizontalTextPosition(JButton.CENTER);
    Send.setVerticalTextPosition(JButton.BOTTOM);
    Send.setToolTipText("Send Faxnumber to Server");
    Send.setMnemonic('S');
    Send.setEnabled(false);
    ImageIcon skullImage = new ImageIcon("skull.gif");
    Quit = new JButton("Quit", skullImage);
    Quit.setHorizontalTextPosition(JButton.CENTER);
    Quit.setVerticalTextPosition(JButton.BOTTOM);
    Quit.setToolTipText("Exit FaxClient");
    Quit.setMnemonic('Q');
    ImageIcon bookImage = new ImageIcon("book.gif");
    Add = new JButton("Add", bookImage);
    Add.setHorizontalTextPosition(JButton.CENTER);
    Add.setVerticalTextPosition(JButton.BOTTOM);
    Add.setToolTipText("Add current entry to Phonebook");
    Add.setMnemonic('A');
    ImageIcon trashImage = new ImageIcon("trash.gif");
    Remove = new JButton("Remove", trashImage);
    Remove.setHorizontalTextPosition(JButton.CENTER);
    Remove.setVerticalTextPosition(JButton.BOTTOM);
    Remove.setToolTipText("Remove current entry from Phonebook");
    Remove.setMnemonic('R');

    receiverCB.setEditable(true);
    receiverCB.setRenderer(new PhoneCellRenderer());
    receiverCB.setSelectedItem("Noname");
    loginTF.setText(System.getProperty("user.name"));
    delayTimeTF.setText("18:00");
    delayTimeTF.setColumns(5);

    /* Create Thread that waits for TCP/IP connection */
    listenThread = new Thread(FaxClient.this);
    listenThread.start();

    /* Create Logo of FaxClient */
    ImageIcon logoImage = new ImageIcon("logo.jpg");
    JLabel logoButton = new JLabel("", logoImage, JLabel.CENTER);
    mainFrame.getContentPane().add("North", logoButton);

    /* Create the Settingbox and its subboxes */
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    JPanel textPanel = new JPanel(gb);
    textPanel.setBorder(BorderFactory.createTitledBorder("Settings"));    
    textPanel.setAlignmentX(JPanel.CENTER_ALIGNMENT);
    mainFrame.getContentPane().add("Center",textPanel);
    JPanel phonePanel = new JPanel(new GridLayout(1, 2, 5, 5));
    phonePanel.setBorder(BorderFactory.createTitledBorder("Phonebook"));    
    phonePanel.setAlignmentX(JPanel.CENTER_ALIGNMENT);
    JPanel delayPanel = new JPanel();
    delayPanel.setBorder(BorderFactory.createTitledBorder("Delay Fax Send"));
    delayPanel.setAlignmentX(JPanel.CENTER_ALIGNMENT);

    gbc.insets = new Insets(5,5,5,5);
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(l1, gbc);
    textPanel.add(l1);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gb.setConstraints(receiverCB, gbc);
    textPanel.add(receiverCB);
    gbc.gridwidth = 1;
    gb.setConstraints(l2, gbc);
    textPanel.add(l2);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gb.setConstraints(faxTF, gbc);
    textPanel.add(faxTF);
    gbc.gridwidth = 1;
    gb.setConstraints(l3, gbc);
    textPanel.add(l3);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gb.setConstraints(loginTF, gbc);
    textPanel.add(loginTF);
    gbc.gridwidth = 1;
    gb.setConstraints(l4, gbc);
    textPanel.add(l4);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gb.setConstraints(nameTF, gbc);
    textPanel.add(nameTF);
    gb.setConstraints(phonePanel, gbc);
    textPanel.add(phonePanel);
    phonePanel.add(Add);
    phonePanel.add(Remove);
    gb.setConstraints(delayPanel, gbc);
    textPanel.add(delayPanel);
    delayPanel.add(delayEnableCB);
    delayPanel.add(l5);
    delayPanel.add(delayTimeTF);

    /* Create Send and Quit button Panel */
    JPanel buttonPanel = new JPanel(new GridLayout(1, 3, 5, 5));
    mainFrame.getContentPane().add("South", buttonPanel);
    buttonPanel.add(Send);
    buttonPanel.add(Quit);

    /* Add all event listener for buttons etc. */
    Add.addActionListener(new ABL());
    Remove.addActionListener(new RBL());
    Send.addActionListener(new SBL());
    Quit.addActionListener(new QBL());
    receiverCB.addItemListener(new IL());
    mainFrame.addWindowListener(new WL());

    /* Set size of Main Window (with a little correction) */
    mainFrame.pack();
    LoadPhonebook();
  }

  public void run() {
    try {
      Thread.currentThread().sleep(1500); // workaround against BindException
      ServerSocket s = new ServerSocket(DEFAULT_PORT);
      socket = s.accept();
      Send.setEnabled(true);
      mainFrame.setVisible(true);
    } catch(Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }

  static class WL extends WindowAdapter {
    public void winodwClosing(WindowEvent we) {
      System.exit(0);
    }
  }

  class IL implements ItemListener {
    public void itemStateChanged(ItemEvent ie) {
      if (receiverCB.getSelectedItem() instanceof PhoneEntry) {
        faxTF.setText(((PhoneEntry)receiverCB.getSelectedItem()).fax);
      }
    }
  }

  class ABL implements ActionListener {
    public void actionPerformed(ActionEvent ae) {
      PhoneEntry ni = new PhoneEntry();
      ni.name = receiverCB.getSelectedItem().toString();
      ni.fax = faxTF.getText();
      receiverCB.addItem(ni);
      SavePhonebook();
    }
  }

  class RBL implements ActionListener {
    public void actionPerformed(ActionEvent ae) {
      receiverCB.removeItem(receiverCB.getSelectedItem());
      SavePhonebook();
    }
  }

  class QBL implements ActionListener {
    public void actionPerformed(ActionEvent ae) {
      System.exit(0);
    }
  }

  class SBL implements ActionListener {
    public void actionPerformed(ActionEvent ae) {
      if (socket == null) return;
      if (faxTF.getText().equals("")) {
	JOptionPane.showMessageDialog
	  (null, "No Faxnumber specified!", "Warning",
	   JOptionPane.WARNING_MESSAGE);
       	return;
      }
      try {
        PrintWriter out =
          new PrintWriter(
            new BufferedWriter(
              new OutputStreamWriter(
                socket.getOutputStream())), true);
        out.println(faxTF.getText());
        out.println(loginTF.getText());
	out.println(receiverCB.getSelectedItem());
        out.println(nameTF.getText());
        if (delayEnableCB.isSelected()) {
	  if (delayTimeTF.getText().equals("")) out.println("delayed");
	  else out.println(delayTimeTF.getText());
        } else out.println("");
        out.close();
        socket.close();
      } catch(Exception e) {
        e.printStackTrace();
      }
      Send.setEnabled(false);
      mainFrame.setVisible(false);
      if (listenThread != null) listenThread.stop();
      listenThread = new Thread(FaxClient.this);
      listenThread.start();
    }
  }

  class PhoneCellRenderer extends JLabel implements ListCellRenderer
  {
    ImageIcon PhoneGroupImage = new ImageIcon("group.gif");
    ImageIcon PhoneSingleImage = new ImageIcon("single.gif");

    private boolean isGroupEntry(String faxnums)
    {
      if((faxnums.indexOf(" ") > -1) || (faxnums.indexOf(",") > -1))
        return true;
      else
        return false;
    }
    public PhoneCellRenderer()
    {
      setOpaque(true);
    }
    public Component getListCellRendererComponent(
          JList listbox, 
          Object value, 
          int index, 
          boolean isSelected, 
          boolean cellHasFocus) 
    {
      PhoneEntry pe = (PhoneEntry) value;
      if(UIManager.getLookAndFeel().getName().equals("CDE/Motif")) {
        if(index == -1 )
          setOpaque(false);
        else
          setOpaque(true);
      } else 
        setOpaque(true);
        if(value == null) {
          setText("");
	  setIcon(null);
      } else if(isSelected) {
        setBackground(UIManager.getColor("ComboBox.selectedBackground"));
        setForeground(UIManager.getColor("ComboBox.selectedForeground"));
        if(isGroupEntry(pe.fax))
          setIcon(PhoneGroupImage);
        else
          setIcon(PhoneSingleImage);
        setText(pe.toString());
      } else {
        if(isGroupEntry(pe.fax))
          setIcon(PhoneGroupImage);
        else
          setIcon(PhoneSingleImage);
        setText(pe.toString());
        setBackground(UIManager.getColor("ComboBox.background"));
        setForeground(UIManager.getColor("ComboBox.foreground"));
      }
      return this;
    }
  }

  /* Load phonebook entries from a file */
  private void LoadPhonebook() {
    try {
      File f = new File(fileName);
      FileInputStream fis = new FileInputStream(f);
      DataInputStream dis = new DataInputStream(fis);
      int j = dis.readInt();
      for(int i = 0; i < j; i++) {
	PhoneEntry k = new PhoneEntry();
	k.name = dis.readUTF();
        k.fax = dis.readUTF();
	receiverCB.addItem(k);
      }
      dis.close();
      fis.close();
    } catch(IOException e) {
      // do nothing
    }
  }

  /* Save all entries in a file */
  private void SavePhonebook() {
    try {
      File f = new File(fileName);
      FileOutputStream fos = new FileOutputStream(f);
      DataOutputStream dos = new DataOutputStream(fos);
      dos.writeInt(receiverCB.getItemCount());
      for(int i = 0; i < receiverCB.getItemCount(); i++) {
	if (receiverCB.getItemAt(i) instanceof PhoneEntry) {
	  dos.writeUTF(((PhoneEntry)receiverCB.getItemAt(i)).name);
	  dos.writeUTF(((PhoneEntry)receiverCB.getItemAt(i)).fax);
	}
      }
      dos.close();
      fos.close();
    } catch(IOException e) {
      e.printStackTrace();
    }
  }

  /* Start for JVM - just create an instance of FaxClient */
  public static void main(String args[]) {
    if (args.length == 0) fileName = "phonebook.dat";
    else fileName = args[0];
    FaxClient fc = new FaxClient();
    fc.init();
  }
}
